@@ -1,3 +1,4 @@
+# encoding: utf-8
 require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
 require ENV['SEQUEL_MSSQL_SPEC_REQUIRE'] if ENV['SEQUEL_MSSQL_SPEC_REQUIRE']
@@ -34,7 +35,7 @@ MSSQL_DB.create_table! :test4 do
   varbinary :value
 end
 
-context "A MSSQL database" do
+describe "A MSSQL database" do
   before do
     @db = MSSQL_DB
   end
@@ -59,19 +60,48 @@ context "A MSSQL database" do
   end
 end
 
-context "MSSQL Dataset#join_table" do
+# This spec is currently disabled as the SQL Server 2008 R2 Express doesn't support
+# full text searching.  Even if full text searching is supported,
+# you may need to create a full text catalog on the database first via:
+#   CREATE FULLTEXT CATALOG ftscd AS DEFAULT
+describe "MSSQL full_text_search" do
+  before do
+    @db = MSSQL_DB
+    @db.drop_table?(:posts)
+  end
+  after do
+    @db.drop_table?(:posts)
+  end
+  
+  specify "should support fulltext indexes and full_text_search" do
+    log do
+      @db.create_table(:posts){Integer :id, :null=>false; String :title; String :body; index :id, :name=>:fts_id_idx, :unique=>true; full_text_index :title, :key_index=>:fts_id_idx; full_text_index [:title, :body], :key_index=>:fts_id_idx}
+      @db[:posts].insert(:title=>'ruby rails', :body=>'y')
+      @db[:posts].insert(:title=>'sequel', :body=>'ruby')
+      @db[:posts].insert(:title=>'ruby scooby', :body=>'x')
+
+      @db[:posts].full_text_search(:title, 'rails').all.should == [{:title=>'ruby rails', :body=>'y'}]
+      @db[:posts].full_text_search([:title, :body], ['sequel', 'ruby']).all.should == [{:title=>'sequel', :body=>'ruby'}]
+
+      @db[:posts].full_text_search(:title, :$n).call(:select, :n=>'rails').should == [{:title=>'ruby rails', :body=>'y'}]
+      @db[:posts].full_text_search(:title, :$n).prepare(:select, :fts_select).call(:n=>'rails').should == [{:title=>'ruby rails', :body=>'y'}]
+    end
+  end
+end if false
+
+describe "MSSQL Dataset#join_table" do
   specify "should emulate the USING clause with ON" do
     MSSQL_DB[:items].join(:categories, [:id]).sql.should ==
-      'SELECT * FROM ITEMS INNER JOIN CATEGORIES ON (CATEGORIES.ID = ITEMS.ID)'
-    ['SELECT * FROM ITEMS INNER JOIN CATEGORIES ON ((CATEGORIES.ID1 = ITEMS.ID1) AND (CATEGORIES.ID2 = ITEMS.ID2))',
-     'SELECT * FROM ITEMS INNER JOIN CATEGORIES ON ((CATEGORIES.ID2 = ITEMS.ID2) AND (CATEGORIES.ID1 = ITEMS.ID1))'].
-    should include(MSSQL_DB[:items].join(:categories, [:id1, :id2]).sql)
+      'SELECT * FROM [ITEMS] INNER JOIN [CATEGORIES] ON ([CATEGORIES].[ID] = [ITEMS].[ID])'
+    ['SELECT * FROM [ITEMS] INNER JOIN [CATEGORIES] ON (([CATEGORIES].[ID1] = [ITEMS].[ID1]) AND ([CATEGORIES].[ID2] = [ITEMS].[ID2]))',
+      'SELECT * FROM [ITEMS] INNER JOIN [CATEGORIES] ON (([CATEGORIES].[ID2] = [ITEMS].[ID2]) AND ([CATEGORIES].[ID1] = [ITEMS].[ID1]))'].
+      should include(MSSQL_DB[:items].join(:categories, [:id1, :id2]).sql)
     MSSQL_DB[:items___i].join(:categories___c, [:id]).sql.should ==
-      'SELECT * FROM ITEMS AS I INNER JOIN CATEGORIES AS C ON (C.ID = I.ID)'
+      'SELECT * FROM [ITEMS] AS [I] INNER JOIN [CATEGORIES] AS [C] ON ([C].[ID] = [I].[ID])'
   end
 end
 
-context "MSSQL Dataset#output" do
+describe "MSSQL Dataset#output" do
   before do
     @db = MSSQL_DB
     @db.create_table!(:items){String :name; Integer :value}
@@ -79,50 +109,49 @@ context "MSSQL Dataset#output" do
     @ds = @db[:items]
   end
   after do
-    @db.drop_table(:items)
-    @db.drop_table(:out)
+    @db.drop_table?(:items, :out)
   end
 
   specify "should format OUTPUT clauses without INTO for DELETE statements" do
     @ds.output(nil, [:deleted__name, :deleted__value]).delete_sql.should =~
-      /DELETE FROM ITEMS OUTPUT DELETED.(NAME|VALUE), DELETED.(NAME|VALUE)/
+      /DELETE FROM \[ITEMS\] OUTPUT \[DELETED\].\[(NAME|VALUE)\], \[DELETED\].\[(NAME|VALUE)\]/
     @ds.output(nil, [:deleted.*]).delete_sql.should =~
-      /DELETE FROM ITEMS OUTPUT DELETED.*/
+      /DELETE FROM \[ITEMS\] OUTPUT \[DELETED\].*/
   end
   
   specify "should format OUTPUT clauses with INTO for DELETE statements" do
     @ds.output(:out, [:deleted__name, :deleted__value]).delete_sql.should =~
-      /DELETE FROM ITEMS OUTPUT DELETED.(NAME|VALUE), DELETED.(NAME|VALUE) INTO OUT/
+      /DELETE FROM \[ITEMS\] OUTPUT \[DELETED\].\[(NAME|VALUE)\], \[DELETED\].\[(NAME|VALUE)\] INTO \[OUT\]/
     @ds.output(:out, {:name => :deleted__name, :value => :deleted__value}).delete_sql.should =~
-      /DELETE FROM ITEMS OUTPUT DELETED.(NAME|VALUE), DELETED.(NAME|VALUE) INTO OUT \((NAME|VALUE), (NAME|VALUE)\)/
+      /DELETE FROM \[ITEMS\] OUTPUT \[DELETED\].\[(NAME|VALUE)\], \[DELETED\].\[(NAME|VALUE)\] INTO \[OUT\] \(\[(NAME|VALUE)\], \[(NAME|VALUE)\]\)/
   end
 
   specify "should format OUTPUT clauses without INTO for INSERT statements" do
     @ds.output(nil, [:inserted__name, :inserted__value]).insert_sql(:name => "name", :value => 1).should =~
-      /INSERT INTO ITEMS \((NAME|VALUE), (NAME|VALUE)\) OUTPUT INSERTED.(NAME|VALUE), INSERTED.(NAME|VALUE) VALUES \((N'name'|1), (N'name'|1)\)/
+      /INSERT INTO \[ITEMS\] \(\[(NAME|VALUE)\], \[(NAME|VALUE)\]\) OUTPUT \[INSERTED\].\[(NAME|VALUE)\], \[INSERTED\].\[(NAME|VALUE)\] VALUES \((N'name'|1), (N'name'|1)\)/
     @ds.output(nil, [:inserted.*]).insert_sql(:name => "name", :value => 1).should =~
-      /INSERT INTO ITEMS \((NAME|VALUE), (NAME|VALUE)\) OUTPUT INSERTED.* VALUES \((N'name'|1), (N'name'|1)\)/
+      /INSERT INTO \[ITEMS\] \(\[(NAME|VALUE)\], \[(NAME|VALUE)\]\) OUTPUT \[INSERTED\].* VALUES \((N'name'|1), (N'name'|1)\)/
   end
 
   specify "should format OUTPUT clauses with INTO for INSERT statements" do
     @ds.output(:out, [:inserted__name, :inserted__value]).insert_sql(:name => "name", :value => 1).should =~
-      /INSERT INTO ITEMS \((NAME|VALUE), (NAME|VALUE)\) OUTPUT INSERTED.(NAME|VALUE), INSERTED.(NAME|VALUE) INTO OUT VALUES \((N'name'|1), (N'name'|1)\)/
+      /INSERT INTO \[ITEMS\] \((\[NAME\]|\[VALUE\]), (\[NAME\]|\[VALUE\])\) OUTPUT \[INSERTED\].\[(NAME|VALUE)\], \[INSERTED\].\[(NAME|VALUE)\] INTO \[OUT\] VALUES \((N'name'|1), (N'name'|1)\)/
     @ds.output(:out, {:name => :inserted__name, :value => :inserted__value}).insert_sql(:name => "name", :value => 1).should =~
-      /INSERT INTO ITEMS \((NAME|VALUE), (NAME|VALUE)\) OUTPUT INSERTED.(NAME|VALUE), INSERTED.(NAME|VALUE) INTO OUT \((NAME|VALUE), (NAME|VALUE)\) VALUES \((N'name'|1), (N'name'|1)\)/
+      /INSERT INTO \[ITEMS\] \(\[(NAME|VALUE)\], \[(NAME|VALUE)\]\) OUTPUT \[INSERTED\].\[(NAME|VALUE)\], \[INSERTED\].\[(NAME|VALUE)\] INTO \[OUT\] \(\[(NAME|VALUE)\], \[(NAME|VALUE)\]\) VALUES \((N'name'|1), (N'name'|1)\)/
   end
 
   specify "should format OUTPUT clauses without INTO for UPDATE statements" do
     @ds.output(nil, [:inserted__name, :deleted__value]).update_sql(:value => 2).should =~
-      /UPDATE ITEMS SET VALUE = 2 OUTPUT (INSERTED.NAME|DELETED.VALUE), (INSERTED.NAME|DELETED.VALUE)/
+      /UPDATE \[ITEMS\] SET \[VALUE\] = 2 OUTPUT \[(INSERTED\].\[NAME|DELETED\].\[VALUE)\], \[(INSERTED\].\[NAME|DELETED\].\[VALUE)\]/
     @ds.output(nil, [:inserted.*]).update_sql(:value => 2).should =~
-      /UPDATE ITEMS SET VALUE = 2 OUTPUT INSERTED.*/
+      /UPDATE \[ITEMS\] SET \[VALUE\] = 2 OUTPUT \[INSERTED\].*/
   end
 
   specify "should format OUTPUT clauses with INTO for UPDATE statements" do
     @ds.output(:out, [:inserted__name, :deleted__value]).update_sql(:value => 2).should =~
-      /UPDATE ITEMS SET VALUE = 2 OUTPUT (INSERTED.NAME|DELETED.VALUE), (INSERTED.NAME|DELETED.VALUE) INTO OUT/
+      /UPDATE \[ITEMS\] SET \[VALUE\] = 2 OUTPUT \[(INSERTED\].\[NAME|DELETED\].\[VALUE)\], \[(INSERTED\].\[NAME|DELETED\].\[VALUE)\] INTO \[OUT\]/
     @ds.output(:out, {:name => :inserted__name, :value => :deleted__value}).update_sql(:value => 2).should =~
-      /UPDATE ITEMS SET VALUE = 2 OUTPUT (INSERTED.NAME|DELETED.VALUE), (INSERTED.NAME|DELETED.VALUE) INTO OUT \((NAME|VALUE), (NAME|VALUE)\)/
+      /UPDATE \[ITEMS\] SET \[VALUE\] = 2 OUTPUT \[(INSERTED\].\[NAME|DELETED\].\[VALUE)\], \[(INSERTED\].\[NAME|DELETED\].\[VALUE)\] INTO \[OUT\] \(\[(NAME|VALUE)\], \[(NAME|VALUE)\]\)/
   end
 
   specify "should execute OUTPUT clauses in DELETE statements" do
@@ -150,34 +179,39 @@ context "MSSQL Dataset#output" do
   end
 end
 
-context "MSSQL dataset" do
+describe "MSSQL dataset" do
   before do
     @db = MSSQL_DB
     @ds = MSSQL_DB[:t]
   end
 
-  context "using #with and #with_recursive" do
+  describe "using #with and #with_recursive" do
     before do
       @ds1 = @ds.with(:t, @db[:x])
       @ds2 = @ds.with_recursive(:t, @db[:x], @db[:t])
     end
 
     specify "should prepend UPDATE statements with WITH clause" do
-      @ds1.update_sql(:x => :y).should == 'WITH T AS (SELECT * FROM X) UPDATE T SET X = Y'
-      @ds2.update_sql(:x => :y).should == 'WITH T AS (SELECT * FROM X UNION ALL SELECT * FROM T) UPDATE T SET X = Y'
+      @ds1.update_sql(:x => :y).should == 'WITH [T] AS (SELECT * FROM [X]) UPDATE [T] SET [X] = [Y]'
+      @ds2.update_sql(:x => :y).should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) UPDATE [T] SET [X] = [Y]'
     end
 
     specify "should prepend DELETE statements with WITH clause" do
-      @ds1.filter(:y => 1).delete_sql.should == 'WITH T AS (SELECT * FROM X) DELETE FROM T WHERE (Y = 1)'
-      @ds2.filter(:y => 1).delete_sql.should == 'WITH T AS (SELECT * FROM X UNION ALL SELECT * FROM T) DELETE FROM T WHERE (Y = 1)'
+      @ds1.filter(:y => 1).delete_sql.should == 'WITH [T] AS (SELECT * FROM [X]) DELETE FROM [T] WHERE ([Y] = 1)'
+      @ds2.filter(:y => 1).delete_sql.should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) DELETE FROM [T] WHERE ([Y] = 1)'
     end
 
     specify "should prepend INSERT statements with WITH clause" do
-      @ds1.insert_sql(@db[:t]).should == 'WITH T AS (SELECT * FROM X) INSERT INTO T SELECT * FROM T'
-      @ds2.insert_sql(@db[:t]).should == 'WITH T AS (SELECT * FROM X UNION ALL SELECT * FROM T) INSERT INTO T SELECT * FROM T'
+      @ds1.insert_sql(@db[:t]).should == 'WITH [T] AS (SELECT * FROM [X]) INSERT INTO [T] SELECT * FROM [T]'
+      @ds2.insert_sql(@db[:t]).should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) INSERT INTO [T] SELECT * FROM [T]'
     end
 
-    context "on #import" do
+    specify "should move WITH clause on joined dataset to top level" do
+      @db[:s].inner_join(@ds1).sql.should == "WITH [T] AS (SELECT * FROM [X]) SELECT * FROM [S] INNER JOIN (SELECT * FROM [T]) AS [T1]"
+      @ds1.inner_join(@db[:s].with(:s, @db[:y])).sql.should == "WITH [T] AS (SELECT * FROM [X]), [S] AS (SELECT * FROM [Y]) SELECT * FROM [T] INNER JOIN (SELECT * FROM [S]) AS [T1]"
+    end
+
+    describe "on #import" do
       before do
         @db = @db.clone
         class << @db
@@ -202,10 +236,10 @@ context "MSSQL dataset" do
         @db[:items].with(:items, @db[:inventory].group(:type)).import([:x, :y], [[1, 2], [3, 4], [5, 6]], :slice => 2)
         @db.import_sqls.should == [
           'BEGIN',
-          "WITH ITEMS AS (SELECT * FROM INVENTORY GROUP BY TYPE) INSERT INTO ITEMS (X, Y) SELECT 1, 2 UNION ALL SELECT 3, 4",
+          "WITH [ITEMS] AS (SELECT * FROM [INVENTORY] GROUP BY [TYPE]) INSERT INTO [ITEMS] ([X], [Y]) SELECT 1, 2 UNION ALL SELECT 3, 4",
           'COMMIT',
           'BEGIN',
-          "WITH ITEMS AS (SELECT * FROM INVENTORY GROUP BY TYPE) INSERT INTO ITEMS (X, Y) SELECT 5, 6",
+          "WITH [ITEMS] AS (SELECT * FROM [INVENTORY] GROUP BY [TYPE]) INSERT INTO [ITEMS] ([X], [Y]) SELECT 5, 6",
           'COMMIT'
         ]
       end
@@ -213,19 +247,36 @@ context "MSSQL dataset" do
   end
 end
 
-context "MSSQL joined datasets" do
+describe "MSSQL::Dataset#import" do
+  before do
+    @db = MSSQL_DB
+    @db.create_table!(:test){primary_key :x; Integer :y}
+    @db.sqls.clear
+    @ds = @db[:test]
+  end
+  after do
+    @db.drop_table?(:test)
+  end
+  
+  specify "#import should work correctly with an arbitrary output value" do
+    @ds.output(nil, [:inserted__y, :inserted__x]).import([:y], [[3], [4]]).should == [{:y=>3, :x=>1}, {:y=>4, :x=>2}]
+    @ds.all.should == [{:x=>1, :y=>3}, {:x=>2, :y=>4}]
+  end
+end
+
+describe "MSSQL joined datasets" do
   before do
     @db = MSSQL_DB
   end
 
   specify "should format DELETE statements" do
     @db[:t1].inner_join(:t2, :t1__pk => :t2__pk).delete_sql.should ==
-      "DELETE FROM T1 FROM T1 INNER JOIN T2 ON (T1.PK = T2.PK)"
+      "DELETE FROM [T1] FROM [T1] INNER JOIN [T2] ON ([T1].[PK] = [T2].[PK])"
   end
 
   specify "should format UPDATE statements" do
     @db[:t1].inner_join(:t2, :t1__pk => :t2__pk).update_sql(:pk => :t2__pk).should ==
-      "UPDATE T1 SET PK = T2.PK FROM T1 INNER JOIN T2 ON (T1.PK = T2.PK)"
+      "UPDATE [T1] SET [PK] = [T2].[PK] FROM [T1] INNER JOIN [T2] ON ([T1].[PK] = [T2].[PK])"
   end
 end
 
@@ -239,7 +290,7 @@ describe "Offset support" do
     @ds.import [:id, :parent_id], [[1,nil],[2,nil],[3,1],[4,1],[5,3],[6,5]]
   end
   after do
-    @db.drop_table(:i)
+    @db.drop_table?(:i)
   end
   
   specify "should return correct rows" do
@@ -262,8 +313,7 @@ describe "Common Table Expressions" do
     @ds.import [:id, :parent_id], [[1,nil],[2,nil],[3,1],[4,1],[5,3],[6,5]]
   end
   after do
-    @db.drop_table(:i1)
-    @db.drop_table(:i2)
+    @db.drop_table?(:i1, :i2)
   end
 
   specify "using #with should be able to update" do
@@ -325,7 +375,7 @@ describe "Common Table Expressions" do
   end
 end
 
-context "MSSSQL::Dataset#insert" do
+describe "MSSSQL::Dataset#insert" do
   before do
     @db = MSSQL_DB
     @db.create_table!(:test5){primary_key :xid; Integer :value}
@@ -333,7 +383,7 @@ context "MSSSQL::Dataset#insert" do
     @ds = @db[:test5]
   end
   after do
-    @db.drop_table(:test5) rescue nil
+    @db.drop_table?(:test5)
   end
 
   specify "should have insert_select return nil if disable_insert_output is used" do
@@ -352,19 +402,19 @@ context "MSSSQL::Dataset#insert" do
   end
 end
 
-context "MSSSQL::Dataset#disable_insert_output" do
+describe "MSSSQL::Dataset#disable_insert_output" do
   specify "should play nicely with simple_select_all?" do
     MSSQL_DB[:test].disable_insert_output.send(:simple_select_all?).should == true
   end
 end
 
-context "MSSSQL::Dataset#into" do
+describe "MSSSQL::Dataset#into" do
   before do
     @db = MSSQL_DB
   end
 
   specify "should format SELECT statement" do
-    @db[:t].into(:new).select_sql.should == "SELECT * INTO NEW FROM T"
+    @db[:t].into(:new).select_sql.should == "SELECT * INTO [NEW] FROM [T]"
   end
 
   specify "should select rows into a new table" do
@@ -372,17 +422,16 @@ context "MSSSQL::Dataset#into" do
     @db[:t].insert(:id => 1, :value => "test")
     @db << @db[:t].into(:new).select_sql
     @db[:new].all.should == [{:id => 1, :value => "test"}]
-    @db.drop_table(:t)
-    @db.drop_table(:new)
+    @db.drop_table?(:t, :new)
   end
 end
 
-context "A MSSQL database" do
+describe "A MSSQL database" do
   before do
     @db = MSSQL_DB
   end
   after do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   specify "should handle many existing types for set_column_allow_null" do
@@ -403,13 +452,13 @@ context "A MSSQL database" do
   end
 end
 
-context "MSSQL::Database#rename_table" do
+describe "MSSQL::Database#rename_table" do
   specify "should work on non-schema bound tables which need escaping" do
     MSSQL_DB.quote_identifiers = true
     MSSQL_DB.create_table! :'foo bar' do
       text :name
     end
-    MSSQL_DB.drop_table :baz rescue nil
+    MSSQL_DB.drop_table? :baz
     proc { MSSQL_DB.rename_table 'foo bar', 'baz' }.should_not raise_error
   end
   
@@ -426,7 +475,7 @@ context "MSSQL::Database#rename_table" do
   end
 end
 
-context "MSSQL::Dataset#count" do
+describe "MSSQL::Dataset#count" do
   specify "should work with a distinct query with an order clause" do
     MSSQL_DB.create_table!(:items){String :name; Integer :value}
     MSSQL_DB[:items].insert(:name => "name", :value => 1)
@@ -435,3 +484,61 @@ context "MSSQL::Dataset#count" do
     MSSQL_DB[:items].select(:name, :value).group(:name, :value).order(:name).count.should == 1
   end
 end
+
+describe "MSSQL::Database#create_table" do
+  specify "should support collate with various other column options" do
+    MSSQL_DB.create_table!(:items){ String :name, :size => 128, :collate => :sql_latin1_general_cp1_ci_as, :default => 'foo', :null => false, :unique => true}
+    MSSQL_DB[:items].insert
+    MSSQL_DB[:items].select_map(:name).should == ["foo"]
+  end
+end
+
+describe "MSSQL::Database#mssql_unicode_strings = false" do
+  before do
+    MSSQL_DB.mssql_unicode_strings = false
+  end
+  after do
+    MSSQL_DB.drop_table?(:items)
+    MSSQL_DB.mssql_unicode_strings = true
+  end
+
+  specify "should work correctly" do
+    MSSQL_DB.create_table!(:items){String :name}
+    MSSQL_DB[:items].mssql_unicode_strings.should == false
+    MSSQL_DB[:items].insert(:name=>'foo')
+    MSSQL_DB[:items].select_map(:name).should == ['foo']
+  end
+
+  specify "should be overridable at the dataset level" do
+    MSSQL_DB.create_table!(:items){String :name}
+    ds = MSSQL_DB[:items]
+    ds.mssql_unicode_strings.should == false
+    ds.mssql_unicode_strings = true
+    ds.mssql_unicode_strings.should == true
+    ds.insert(:name=>'foo')
+    ds.select_map(:name).should == ['foo']
+  end
+end
+
+describe "A MSSQL database adds index with include" do
+  before :all do
+    @table_name = :test_index_include
+    @db = MSSQL_DB
+    @db.create_table! @table_name do
+      integer :col1
+      integer :col2
+      integer :col3
+    end
+  end
+
+  after :all do
+    @db.drop_table? @table_name
+  end
+
+  cspecify "should be able add index with include" do
+    @db.alter_table @table_name do
+      add_index [:col1], :include => [:col2,:col3]
+    end
+    @db.indexes(@table_name).should have_key("#{@table_name}_col1_index".to_sym)
+  end
+end
