@@ -1,6 +1,5 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
-if INTEGRATION_DB.respond_to?(:schema_parse_table, true)
 describe "Database schema parser" do
   before do
     @iom = INTEGRATION_DB.identifier_output_method
@@ -13,7 +12,7 @@ describe "Database schema parser" do
     INTEGRATION_DB.identifier_input_method = @iim
     INTEGRATION_DB.default_schema = @defsch
     INTEGRATION_DB.quote_identifiers = @qi
-    INTEGRATION_DB.drop_table(:items) if INTEGRATION_DB.table_exists?(:items)
+    INTEGRATION_DB.drop_table?(:items)
   end
 
   specify "should handle a database with a identifier methods" do
@@ -137,17 +136,20 @@ describe "Database schema parser" do
     INTEGRATION_DB.create_table!(:items){FalseClass :number}
     INTEGRATION_DB.schema(:items).first.last[:type].should == :boolean
   end
-end
-end
+end if INTEGRATION_DB.respond_to?(:schema_parse_table, true)
 
-begin
-  INTEGRATION_DB.drop_table(:blah) rescue nil
+test_indexes = begin
+  INTEGRATION_DB.drop_table?(:blah)
   INTEGRATION_DB.indexes(:blah)
+  true
 rescue Sequel::NotImplemented
+  false
 rescue
+  true
+end
 describe "Database index parsing" do
   after do
-    INTEGRATION_DB.drop_table(:items)
+    INTEGRATION_DB.drop_table?(:items)
   end
 
   specify "should parse indexes into a hash" do
@@ -173,8 +175,63 @@ describe "Database index parsing" do
     INTEGRATION_DB.create_table!(:items){Integer :n; Integer :a; primary_key [:n, :a]}
     INTEGRATION_DB.indexes(:items).should == {}
   end
+end if test_indexes
+
+test_foreign_key_list = begin
+  INTEGRATION_DB.drop_table?(:blah)
+  INTEGRATION_DB.foreign_key_list(:blah)
+  true
+rescue Sequel::NotImplemented
+  false
+rescue
+  true
 end
-end
+describe "Database foreign key parsing" do
+  before do
+    @db = INTEGRATION_DB
+    @pr = lambda do |table, *expected|
+      actual = @db.foreign_key_list(table).sort_by{|c| c[:columns].map{|s| s.to_s}.join << (c[:key]||[]).map{|s| s.to_s}.join}.map{|v| v.values_at(:columns, :table, :key)}
+      actual.zip(expected).each do |a, e|
+        if e.last.first == :pk
+          if a.last == nil
+            a.pop
+            e.pop
+          else
+           e.last.shift
+          end
+        end
+        a.should == e
+      end
+      actual.length.should == expected.length
+    end
+  end
+  after do
+    @db.drop_table?(:b, :a)
+  end
+
+  specify "should parse foreign key information into an array of hashes" do
+    @db.create_table!(:a, :engine=>:InnoDB){primary_key :c; Integer :d; index :d, :unique=>true}
+    @db.create_table!(:b, :engine=>:InnoDB){foreign_key :e, :a}
+    @pr[:a]
+    @pr[:b, [[:e], :a, [:pk, :c]]]
+
+    @db.alter_table(:b){add_foreign_key :f, :a, :key=>[:d]}
+    @pr[:b, [[:e], :a, [:pk, :c]], [[:f], :a, [:d]]]
+
+    @db.alter_table(:b){add_foreign_key [:f], :a, :key=>[:c]}
+    @pr[:b, [[:e], :a, [:pk, :c]], [[:f], :a, [:c]], [[:f], :a, [:d]]]
+
+    @db.alter_table(:a){add_index [:d, :c], :unique=>true}
+    @db.alter_table(:b){add_foreign_key [:f, :e], :a, :key=>[:d, :c]}
+    @pr[:b, [[:e], :a, [:pk, :c]], [[:f], :a, [:c]], [[:f], :a, [:d]], [[:f, :e], :a, [:d, :c]]]
+  end
+
+  specify "should handle composite foreign and primary keys" do
+    @db.create_table!(:a, :engine=>:InnoDB){Integer :b; Integer :c; primary_key [:b, :c]; index [:c, :b], :unique=>true}
+    @db.create_table!(:b, :engine=>:InnoDB){Integer :e; Integer :f; foreign_key [:e, :f], :a; foreign_key [:f, :e], :a, :key=>[:c, :b]}
+    @pr[:b, [[:e, :f], :a, [:pk, :b, :c]], [[:f, :e], :a, [:c, :b]]]
+  end
+end if test_foreign_key_list
 
 describe "Database schema modifiers" do
   before do
@@ -182,7 +239,7 @@ describe "Database schema modifiers" do
     @ds = @db[:items]
   end
   after do
-    @db.drop_table(:items) if @db.table_exists?(:items)
+    @db.drop_table?(:items, :items2)
   end
 
   specify "should create tables correctly" do
@@ -193,6 +250,36 @@ describe "Database schema modifiers" do
     @ds.columns!.should == [:number]
   end
   
+  specify "should create tables from select statements correctly" do
+    @db.create_table!(:items){Integer :number}
+    @ds.insert([10])
+    @db.create_table(:items2, :as=>@db[:items])
+    @db.schema(:items2, :reload=>true).map{|x| x.first}.should == [:number]
+    @db[:items2].columns.should == [:number]
+    @db[:items2].all.should == [{:number=>10}]
+  end
+  
+  specify "should handle create table in a rolled back transaction" do
+    @db.drop_table?(:items)
+    @db.transaction(:rollback=>:always){@db.create_table(:items){Integer :number}}
+    @db.table_exists?(:items).should be_false
+  end if INTEGRATION_DB.supports_transactional_ddl?
+  
+  describe "join tables" do
+    after do
+      @db.drop_join_table(:cat_id=>:cats, :dog_id=>:dogs) if @db.table_exists?(:cats_dogs)
+      @db.drop_table(:cats, :dogs)
+      @db.table_exists?(:cats_dogs).should == false
+    end
+
+    specify "should create join tables correctly" do
+      @db.create_table!(:cats){primary_key :id}
+      @db.create_table!(:dogs){primary_key :id}
+      @db.create_join_table(:cat_id=>:cats, :dog_id=>:dogs)
+      @db.table_exists?(:cats_dogs).should == true
+    end
+  end
+
   specify "should create temporary tables without raising an exception" do
     @db.create_table!(:items, :temp=>true){Integer :number}
   end
@@ -201,7 +288,7 @@ describe "Database schema modifiers" do
     @db.create_table!(:items){String :a}
     @db.create_table?(:items){String :b}
     @db[:items].columns.should == [:a]
-    @db.drop_table(:items) rescue nil
+    @db.drop_table?(:items)
     @db.create_table?(:items){String :b}
     @db[:items].columns.should == [:b]
   end
@@ -210,13 +297,13 @@ describe "Database schema modifiers" do
     @db.create_table!(:items){String :a, :index=>true}
     @db.create_table?(:items){String :b, :index=>true}
     @db[:items].columns.should == [:a]
-    @db.drop_table(:items) rescue nil
+    @db.drop_table?(:items)
     @db.create_table?(:items){String :b, :index=>true}
     @db[:items].columns.should == [:b]
   end
 
   specify "should rename tables correctly" do
-    @db.drop_table(:items) rescue nil
+    @db.drop_table?(:items)
     @db.create_table!(:items2){Integer :number}
     @db.rename_table(:items2, :items)
     @db.table_exists?(:items).should == true
@@ -320,6 +407,21 @@ describe "Database schema modifiers" do
     proc{@ds.insert(:n=>nil)}.should raise_error(Sequel::DatabaseError)
   end
 
+  specify "should rename columns when the table is referenced by a foreign key" do
+    begin
+      @db.create_table!(:itemsfk){primary_key :id; Integer :a}
+      @db.create_table!(:items){foreign_key :id, :itemsfk}
+      @db[:itemsfk].insert(:a=>10)
+      @ds.insert(:id=>1)
+      @db.alter_table(:itemsfk){rename_column :a, :b}
+      @db[:itemsfk].insert(:b=>20)
+      @ds.insert(:id=>2)
+      @db[:itemsfk].select_order_map([:id, :b]).should == [[1, 10], [2, 20]]
+    ensure
+      @db.drop_table(:items, :itemsfk)
+    end
+  end
+
   cspecify "should set column NULL/NOT NULL correctly", [:jdbc, :db2], [:db2] do
     @db.create_table!(:items, :engine=>:InnoDB){Integer :id}
     @ds.insert(:id=>10)
@@ -352,6 +454,35 @@ describe "Database schema modifiers" do
     @ds.all.should == [{:id=>"10"}, {:id=>"20"}]
   end
 
+  cspecify "should set column types without modifying NULL/NOT NULL", [:jdbc, :db2], [:db2], :oracle, :derby do
+    @db.create_table!(:items){Integer :id, :null=>false, :default=>2}
+    proc{@ds.insert(:id=>nil)}.should raise_error(Sequel::DatabaseError)
+    @db.alter_table(:items){set_column_type :id, String}
+    proc{@ds.insert(:id=>nil)}.should raise_error(Sequel::DatabaseError)
+
+    @db.create_table!(:items){Integer :id}
+    @ds.insert(:id=>nil)
+    @db.alter_table(:items){set_column_type :id, String}
+    @ds.insert(:id=>nil)
+    @ds.map(:id).should == [nil, nil]
+  end
+
+  cspecify "should set column types without modifying defaults", [:jdbc, :db2], [:db2], :oracle, :derby do
+    @db.create_table!(:items){Integer :id, :default=>0}
+    @ds.insert
+    @ds.map(:id).should == [0]
+    @db.alter_table(:items){set_column_type :id, String}
+    @ds.insert
+    @ds.map(:id).should == ['0', '0']
+
+    @db.create_table!(:items){String :id, :default=>'a'}
+    @ds.insert
+    @ds.map(:id).should == %w'a'
+    @db.alter_table(:items){set_column_type :id, String, :size=>1}
+    @ds.insert
+    @ds.map(:id).should == %w'a a'
+  end
+
   specify "should add unnamed unique constraints and foreign key table constraints correctly" do
     @db.create_table!(:items, :engine=>:InnoDB){Integer :id; Integer :item_id}
     @db.alter_table(:items) do
@@ -396,7 +527,9 @@ describe "Database schema modifiers" do
   end
 
   cspecify "should remove columns from tables correctly", :h2, :mssql, [:jdbc, :db2], :hsqldb do
-    @db.create_table!(:items) do
+    # MySQL with InnoDB cannot drop foreign key columns unless you know the
+    # name of the constraint, see Bug #14347
+    @db.create_table!(:items, :engine=>:MyISAM) do
       primary_key :id
       String :name
       Integer :number
@@ -434,10 +567,12 @@ describe "Database schema modifiers" do
   end
 end
 
-begin
+test_tables = begin
   INTEGRATION_DB.tables
+  true
 rescue Sequel::NotImplemented
-rescue
+  false
+end
 describe "Database#tables" do
   before do
     class ::String
@@ -472,13 +607,14 @@ describe "Database#tables" do
     @db.identifier_input_method = :xxxxx
     @db.tables.each{|t| t.to_s.should =~ /\Ax{5}\d+\z/}
   end
-end
-end
+end if test_tables
 
-begin
+test_views = begin
   INTEGRATION_DB.views
+  true
 rescue Sequel::NotImplemented
-rescue
+  false
+end
 describe "Database#views" do
   before do
     class ::String
@@ -513,5 +649,4 @@ describe "Database#views" do
     @db.identifier_input_method = :xxxxx
     @db.views.each{|t| t.to_s.should =~ /\Ax{5}\d+\z/}
   end
-end
-end
+end if test_views
